#| twm-traditional theme.jl

Copyright (c) 2000 Jason F. McBrayer <jmcbray@carcosa.net>
Copyright (c) 1999-2000 John Harper <john@dcs.warwick.ac.uk>

Based on twm theme.jl by John Harper; stricter twm emulation by Jason McBrayer.
Requires sawfish 0.30 upwards.
|#

(defgroup twm-traditional-style "TWM-Traditional style" :group appearance)

(defcustom twm-traditional:title-color (get-color "mediumvioletred")
  "Title color"
  :group (appearance twm-traditional-style)
  :type color
  :after-set (lambda () (rebuild-images) (after-setting-frame-option)))

(defcustom twm-traditional:frame-color (get-color "cadetblue")
  "Frame color"
  :group (appearance twm-traditional-style)
  :type color
  :after-set after-setting-frame-option)

(defcustom twm-traditional:text-color (get-color "black")
  "Text color"
  :group (appearance twm-traditional-style)
  :type color
  :after-set (lambda () (rebuild-images) (after-setting-frame-option)))

(defcustom twm-traditional:squish-titles nil
  "Squish titlebars"
  :group (appearance twm-traditional-style)
  :type boolean
  )



(define stipple-i (make-sized-image 32 32))

(define h-stipple-i (make-sized-image 32 2))

(define v-stipple-i (make-sized-image 2 32))

(define icon-button-i (list (make-image "icon-button.png") nil
                            nil (make-image "icon-button-c.png")))           

(define resize-button-i (list (make-image "resize-button.png") nil
                              nil (make-image "resize-button.png")))
        
        

(define (h-stipple) (list h-stipple-i twm-traditional:frame-color))
(define (v-stipple) (list v-stipple-i twm-traditional:frame-color))

(define (hashed-offset w) (+ (text-width (window-name w)) 24))

(define (title-right-edge w)
  (cond
   (twm-traditional:squish-titles
    (cond
     ((< (car (window-dimensions w)) (+ 128 (text-width (window-name w)))) 0)
     (t (- (car (window-dimensions w)) (+ 128 (text-width (window-name w)))))
     ))
   (t 0)
   ))

(define (right-border-top w)
  (cond
   (twm-traditional:squish-titles -2)
   (t -22)))

(define (rebuild-images)
  (let* ((fg-pixel (nconc (color-rgb-8 twm-traditional:text-color) '(255)))
         (col-white-map
          (vector (nconc (color-rgb-8 twm-traditional:title-color) '(255))
                  fg-pixel)))

    (define (black-white x y)
      (aref [(255 255 255 255) (0 0 0 255)] (logand (logxor x y) 1)))
 
    (define (color-white x y)
      (aref col-white-map (logand (logxor x y) 1)))

    (define (solid->text pixel) (and (> (nth 3 pixel) 0) fg-pixel))
    
    (image-fill black-white h-stipple-i)
    (image-fill black-white v-stipple-i)
    (image-fill color-white stipple-i)

    (mapc (lambda (x)
            (when x
              (image-map solid->text x))) icon-button-i)
    (mapc (lambda (x)
            (when x
              (image-map solid->text x))) resize-button-i)
    ))

(define (update w)
  (if (eq (window-get w 'current-frame-style) 'twm-traditional)
      (rebuild-frame w)))

(define frame 
  `(((foreground . ,(lambda () twm-traditional:text-color))
     (background . ,(lambda () twm-traditional:title-color))
     (text . ,window-name)
     (x-justify . 20)
     (y-justify . center)
     (left-edge . 0)
     (right-edge . ,title-right-edge)
     (top-edge . -20)
     (height . 18)
     (class . title))
    ((background . ,(lambda () twm-traditional:title-color))
     (foreground . ,icon-button-i)
     (left-edge . 2)
     (top-edge . -18)
     (width . 16)
     (height . 16)
     (class . iconify-button))
    ((background . ,(lambda () twm-traditional:title-color))
     (foreground . ,resize-button-i)
     (right-edge . ,(lambda (w) (+ 0 (title-right-edge w))))
     (top-edge . -18)
     (width . 16)
     (height . 16)
     (class . top-border))
    ((background . ,(lambda () (list twm-traditional:title-color stipple-i)))
     (right-edge . ,(lambda (w) (+ 21 (title-right-edge w))))
     (left-edge . ,hashed-offset)
     (top-edge . -19)
     (height . 16)
     (class . title))
    ((background . ,h-stipple)
     (left-edge . 0)
     (right-edge . ,title-right-edge)
     (top-edge . -22)
     (height . 2)
     (class . top-border-inert))
    ((background . ,h-stipple)
     (left-edge . 0)
     (right-edge . 0)
     (top-edge . -2)
     (height . 2))
    ((background . ,h-stipple)
     (left-edge . 0)
     (right-edge . 0)
     (bottom-edge . -2)
     (height . 2)
     (class . bottom-border-inert))
    ((background . ,v-stipple)
     (left-edge . -2)
     (width . 2)
     (top-edge . -22)
     (bottom-edge . -2)
     (class . left-border-inert))
    ((background . ,v-stipple)
     (right-edge . -2)
     (width . 2)
     (top-edge . ,right-border-top)
     (bottom-edge . -2)
     (class . right-border-inert))
    ((background . ,v-stipple)
     (right-edge . ,(lambda (w) (- (title-right-edge w) 2)))
     (width . 2)
     (top-edge . -22)
     (height . 22)
     (class . right-border-inert))
    ))

(define shaped-frame 
  `(((foreground . ,(lambda () twm-traditional:text-color))
     (background . ,(lambda () twm-traditional:title-color))
     (text . ,window-name)
     (x-justify . 20)
     (y-justify . center)
     (left-edge . 0)
     (right-edge . ,title-right-edge)
     (top-edge . -20)
     (height . 18)
     (class . title))
    ((background . ,(lambda ()
                      (list twm-traditional:title-color stipple-i)))
     (right-edge . ,(lambda (w) (+ 3 (title-right-edge w))))
     (left-edge . ,hashed-offset)
     (top-edge . -19)
     (height . 16)
     (class . title))
    ((background . ,h-stipple)
     (left-edge . 0)
     (right-edge . ,title-right-edge)
     (top-edge . -22)
     (height . 2)
     (class . top-border-inert))
    ((background . ,h-stipple)
     (left-edge . 0)
     (right-edge . ,title-right-edge)
     (top-edge . -2)
     (height . 2))
    ((background . ,v-stipple)
     (left-edge . -2)
     (width . 2)
     (top-edge . -22)
     (height . 22)
     (class . left-border-inert))
    ((background . ,v-stipple)
     (right-edge . ,title-right-edge)
     (width . 2)
     (top-edge . -22)
     (height . 22)
     (class . right-border-inert))))

(define transient-frame `(((background . ,h-stipple)
                           (left-edge . 0)
                           (right-edge . 0)
                           (top-edge . -2)
                           (height . 2)
                           (class . title))
                          ((background . ,h-stipple)
                           (left-edge . 0)
                           (right-edge . 0)
                           (bottom-edge . -2)
                           (height . 2)
                           (class . bottom-border-inert))
                          ((background . ,v-stipple)
                           (left-edge . -2)
                           (width . 2)
                           (top-edge . -2)
                           (bottom-edge . -2)
                           (class . left-border-inert))
                          ((background . ,v-stipple)
                           (right-edge . -2)
                           (width . 2)
                           (top-edge . -2)
                           (bottom-edge . -2)
                           (class . right-border-inert))))

(define shaped-transient-frame `(((background . ,h-stipple)
                                  (left-edge . 0)
                                  (right-edge . 0)
                                  (top-edge . -2)
                                  (height . 2)
                                  (class . title))))

(mapc (lambda (i)
        (image-put i 'tiled t))
      (list stipple-i h-stipple-i v-stipple-i))

(add-frame-style 'twm-traditional
                 (lambda (w type)
                   (case type
                     ((default) frame)
                     ((unframed) nil-frame)
                     ((transient) transient-frame)
                     ((shaped-transient) shaped-transient-frame)
                     ((shaped) shaped-frame))))

(rebuild-images)
(call-after-property-changed 'WM_NAME update)
